<?php
/*
Plugin Name: reCaptcha v3
Version: 1.01
Plugin URI: https://nhws.localinfo.jp/
Description: Antispam with the reCaptcha v3 service.
Author: ぴー＠西東京
Author URI: https://nhws.localinfo.jp/
*/


add_action('wp_head', 'plugin_add_recaptchav3_js');
add_action('comment_form', 'plugin_recaptchav3_comment_form');
add_action('comment_validate', 'plugin_recaptchav3_validate', 4, 2);


function plugin_add_recaptchav3_js() {
	$recaptchav3conf = plugin_getoptions('recaptchav3');
	$recaptchav3conf['sitekey'] = isset($recaptchav3conf['sitekey']) ? $recaptchav3conf['sitekey'] : "";
	
	echo "\n" . <<< EOM01
<!-- reCaptcha v3 plugin -->
<script src="https://www.google.com/recaptcha/api.js?render=
EOM01;
	echo $recaptchav3conf['sitekey'] . <<< EOM02
"></script>
<script>
  grecaptcha.ready(function () {
    grecaptcha.execute('
EOM02;
	echo $recaptchav3conf['sitekey'] . <<< EOM03
', { action: 'contact' }).then(function (token) {
      var recaptchaResponse = document.getElementById('recaptchaResponse');
      recaptchaResponse.value = token;
    });
  });
</script>
<!-- end of reCaptcha v3 plugin -->
EOM03;
}


function plugin_recaptchav3_comment_form() {
	echo <<< EOM
<input type="hidden" name="recaptcha_response" id="recaptchaResponse">
EOM;
}


function plugin_recaptchav3_setup() {
	global $fp_config;
	
	if (!plugin_getoptions('recaptchav3','sitekey') && 
			!plugin_getoptions('recaptchav3','secretkey')) {
		return -1;
	}
	return 1;
}


if (class_exists('AdminPanelAction')){
	/*
	Provides an admin panel entry for reCaptcha v3 setup.
	*/
	class admin_plugin_recaptchav3 extends AdminPanelAction { 
		var $langres = 'plugin:recaptchav3';
		
		/*
		Initializes this panel.
		*/
		function setup() {
			$this->smarty->assign('admin_resource', "plugin:recaptchav3/admin.plugin.recaptchav3");
		}
		
		/*
		Setups the default panel.
		*/
		function main() {
			$recaptchav3conf = plugin_getoptions('recaptchav3');
			
			$recaptchav3conf['sitekey'] = isset($recaptchav3conf['sitekey']) ? $recaptchav3conf['sitekey'] : "";
			$recaptchav3conf['secretkey'] = isset($recaptchav3conf['secretkey']) ? $recaptchav3conf['secretkey'] : "";
			$recaptchav3conf['scorethreshold'] = isset($recaptchav3conf['scorethreshold']) ? $recaptchav3conf['scorethreshold'] : 0.5;
			
			$this->smarty->assign('recaptchav3conf', $recaptchav3conf);
		}
		
		/*
		Will be executed when the reCaptcha v3 configuscorethresholdn is send.
		
		@return int
		*/
		function onsubmit($data = null) {
			global $fp_config;
			if ($_POST['sitekey'] && $_POST['secretkey'] && $_POST['scorethreshold']){
				if (!($_POST['scorethreshold'] >= 0 && $_POST['scorethreshold'] <= 1)){
					$_POST['scorethreshold'] = 0.5;
				}
				plugin_addoption('recaptchav3', 'sitekey', $_POST['sitekey']);
				plugin_addoption('recaptchav3', 'secretkey', $_POST['secretkey']);
				plugin_addoption('recaptchav3', 'scorethreshold', $_POST['scorethreshold']);
				plugin_saveoptions('recaptchav3');
				
				$this->smarty->assign('success', 1);
			} else {
				$this->smarty->assign('success', -1);
			}
			return 2;
		}		
	}
	admin_addpanelaction('plugin', 'recaptchav3', true);
}


function plugin_recaptchav3_validate($bool) {
	global $smarty;
	
	// if boolean $bool==false
	// the test is forced to fail
	if (!$bool)
		return false;
	
	//  Check if form was submitted:
	if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['recaptcha_response'])) {
//		global $smarty;
		$lang = lang_load('plugin:recaptchav3');
		$recaptchav3conf = plugin_getoptions('recaptchav3');
		$recaptcha_scorethreshold = isset($recaptchav3conf['scorethreshold']) ? $recaptchav3conf['scorethreshold'] : 0.5;
		
		// Build POST request:
		$recaptcha_url = 'https://www.google.com/recaptcha/api/siteverify';
		$recaptcha_secret = isset($recaptchav3conf['secretkey']) ? $recaptchav3conf['secretkey'] : "";
		$recaptcha_response = htmlspecialchars($_POST['recaptcha_response'], ENT_QUOTES, 'UTF-8');
		
		// Make and decode POST request:
		$recaptcha = file_get_contents($recaptcha_url . '?secret=' . $recaptcha_secret . '&response=' . $recaptcha_response);
		$recaptcha = json_decode($recaptcha);
		if (!$recaptcha) {
			$smarty->append('error', $lang['plugin']['recaptchav3']['error'] . ' : response error');
			return false;
		}
		$recaptcha_score = $recaptcha->score;
//		$recaptcha_error_codes = $recaptcha->error-codes;
		
		// Take action based on the score returned:
		if ($recaptcha_score >= $recaptcha_scorethreshold) {
			// Verified - send email
			return true;
		} else {
			// Not verified - show form error
			$smarty->append('error', $lang['plugin']['recaptchav3']['error'] . ' ' . $recaptcha_score);
			return false;
		}
	} else {
		$smarty->append('error', $lang['plugin']['recaptchav3']['error'] . ' : no token');
		return false;
	}
}


?>
